<?php

namespace Myriad\EventDrivenService\Repositories;

use Carbon\Carbon;
use Myriad\EventDrivenService\Enums\Platform;
use Myriad\EventDrivenService\Http\Utilities\EventHelper;
use Myriad\EventDrivenService\Models\FailedEvent;
use Myriad\EventDrivenService\Traits\Logger;

class FailedEventRepository {
  use Logger;

  public function create($event, $reason) {
    if (isset($event['value']) && is_array($event['value'])) {
      $event['value'] = json_encode($event['value']);
    }
    try {
      Carbon::createFromFormat(EventHelper::OCCURRED_AT_FORMAT, $event['occurred_at']);
    } catch (\Exception $e) {
      unset($event['occurred_at']);
    }
    return FailedEvent::create([
      "command" => array_key_exists('command', $event) ? $event['command'] : null,
      "model_uuid" => array_key_exists('model_uuid', $event) ? $event['model_uuid'] : null,
      "name" => array_key_exists('name', $event) ? $event['name'] : null,
      "value" => array_key_exists('value', $event) ? $event['value'] : null,
      "version" => array_key_exists('version', $event) ? $event['version'] : null,
      "version_new" => array_key_exists('version_new', $event) ? $event['version_new'] : null,
      "reason" => $reason,
      "occurred_at" => array_key_exists('occurred_at', $event) ? Carbon::createFromFormat(EventHelper::OCCURRED_AT_FORMAT, $event['occurred_at'])->tz('utc') : null,
      "user_agent" => request()->header('User-Agent'),
      "app_version" => request()->header('App-Version'),
      "device_model_number" => $this->getDeviceModelNumber(),
      "device_os_version" => $this->getDeviceOSVersion(),
      "release_version" => request()->header('Release-Version'),
      "platform" => $this->getPlatform(),
    ]);
  }

  private function getDeviceModelNumber() {
    if (request()->hasHeader('Model-Number')) {
      return request()->header('Model-Number');
    }

    if (request()->hasHeader('Platform-Name')) {
      return request()->header('Platform-Name');
    }

    return null;
  }

  private function getDeviceOSVersion() {
    if (request()->hasHeader('OS-Version')) {
      return request()->header('OS-Version');
    }

    if (request()->hasHeader('Platform-Version')) {
      return request()->header('Platform-Version');
    }

    return null;
  }

  private function getPlatform() {
    switch (request()->header('User-Agent')) {
      case Platform::ANDROID:
      case Platform::IOS:
        return request()->header('User-Agent');
      default:
        return Platform::WEB;
    }
  }

  public function createAll($errors) {
    foreach ($errors as $error) {
      $this->create($error['request'], $error['error']);
    }
  }
}