<?php

namespace Myriad\EventDrivenService\Traits;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;

trait Logger {

  protected $logEvent;
  protected $logUser;

  protected function logInfo($message) {
    Log::info($message, $this->getMeta());
  }

  protected function logError($message) {
    Log::error($message, $this->getMeta());
  }

  protected function logDebug($message) {
    Log::debug($message, $this->getMeta());
  }

  protected function logJobStarted($logEvent = null, $logUser = null) {
    $this->logTime("Started", $logEvent, $logUser);
  }

  protected function logJobFinished($logEvent = null, $logUser = null) {
    $this->logTime("Finished", $logEvent, $logUser);
  }

  protected function logJobFailed($logEvent = null, $logUser = null) {
    $this->logTime("Failed", $logEvent, $logUser);
  }

  protected function handleVariables($logEvent, $logUser) {
    if ($logEvent) {
      $this->logEvent = $logEvent;
    }
    if ($logUser) {
      $this->logUser = $logUser;
    }
  }

  private function logTime($trigger, $logEvent, $logUser) {
    $this->handleVariables($logEvent, $logUser);
    $time = Carbon::now()->toAtomString();
    $this->logInfo("{$this->logEvent}: {$trigger} at {$time}.");
  }

  private function getMeta() {
    return [
      'user_uuid' => $this->getUser() ? $this->getUser()->uuid : null,
      'user_name' => $this->getUser() ? $this->getUser()->name : null,
      'headers' => Request::capture()->headers->all(),
      'body' => Request::capture()->all(),
    ];
  }

  private function getUser() {
    if (!$this->logUser) {
      $this->logUser = Auth::check() ? Auth::user() : null;
    }
    return $this->logUser;
  }
}
